use nom::combinator::recognize;

use super::object::PlainText;
use super::Context;
use crate::error::CustomError;
use crate::error::MyError;
use crate::error::Res;
use crate::parser::exiting::ExitClass;
use crate::parser::object_parser::any_object_except_plain_text;
use crate::parser::parser_context::ContextElement;
use crate::parser::parser_context::ExitMatcherNode;
use crate::parser::parser_with_context::parser_with_context;
use crate::parser::util::text_until_exit;

#[tracing::instrument(ret, level = "debug")]
pub fn plain_text<'r, 's>(context: Context<'r, 's>, input: &'s str) -> Res<&'s str, PlainText<'s>> {
    if input.len() == 0 {
        return Err(nom::Err::Error(CustomError::MyError(MyError(
            "Zero input length to plain_text.",
        ))));
    }
    let parser_context =
        context.with_additional_node(ContextElement::ExitMatcherNode(ExitMatcherNode {
            class: ExitClass::Beta,
            exit_matcher: &plain_text_end,
        }));
    let (remaining, source) = text_until_exit(&parser_context, input)?;
    Ok((remaining, PlainText { source }))
}

#[tracing::instrument(ret, level = "debug")]
fn plain_text_end<'r, 's>(context: Context<'r, 's>, input: &'s str) -> Res<&'s str, &'s str> {
    recognize(parser_with_context!(any_object_except_plain_text)(context))(input)
}

#[cfg(test)]
mod tests {
    use nom::combinator::map;

    use super::*;
    use crate::parser::object::Object;
    use crate::parser::parser_context::ContextElement;
    use crate::parser::parser_context::ContextTree;
    use crate::parser::parser_with_context::parser_with_context;
    use crate::parser::source::Source;

    #[test]
    fn plain_text_simple() {
        let input = "foobarbaz";
        let initial_context: ContextTree<'_, '_> = ContextTree::new();
        let document_context =
            initial_context.with_additional_node(ContextElement::DocumentRoot(input));
        let plain_text_matcher = parser_with_context!(plain_text)(&document_context);
        let (remaining, result) = map(plain_text_matcher, Object::PlainText)(input).unwrap();
        assert_eq!(remaining, "");
        assert_eq!(result.get_source(), input);
    }
}
